#!/bin/bash
###############################################################################
# Brief     : 指定程序及其对应守护进程的开启或关闭
# Author    : shn
# Date      : 20190116
# Version   : 1.2
# Note      : 注意程序路径和程序名称;
#             注意在daemon.sh中修改程序启动或关闭函数;
#             如果程序有参数,需要修改启动函数;
# 20190328 增加守护进程, ${work_dir}主要用于启动程序及生成日志
###############################################################################


prog_name="RestAPI"             # 视情况修改
t_user=""                       # 默认不检查用户,如果检查注意增加对应的ps管道
work_dir=$(cd $(dirname ${BASH_SOURCE[0]}); pwd)
#work_dir=$(cd `dirname $0` && pwd)
prog_path=${work_dir}/${prog_name}


function start_daemon() {
    local daemon=$1
    local command=$2
    local interval=$3
    [ -z "${interval}" ] && interval=10s
    local daemon_output=$4
    [ -z "${daemon_output}" ] && daemon_output="${work_dir}/output$(date '+%Y%m%d').log"
    # 开启守护进程，如果守护进程已开启，则忽略再次启动
    local num=`ps -ef | grep -v grep | grep daemon | grep -w "${command}" | wc -l`
    if [ "${num}" -lt "1" ]; then
        "${daemon}" "${command}" "${interval}" "${daemon_output}"
        echo -e "the daemon for command [ ${command} ] start success!"
    else 
        echo -e "the daemon for command [ ${command} ] is already running, ignore this start-up procedure"
    fi
}

function stop_daemon(){
    #关闭当前路径下开启的名字为"$prog_name"的进程
    local t_prog_path=$1
    local t_prog_name=$(basename ${t_prog_path})
    local num=`ps -ef | grep -v grep | grep ${t_prog_path} | wc -l`
    if [ ${num} -ge 1 ];then
        # 关闭守护进程
        ps -ef | grep -v grep | grep daemon | grep ${t_prog_path} | awk '{print $2}' | xargs kill -9 >/dev/null 2>&1
        echo -e "\e[31;1mthe daemon for command [ ${t_prog_path} ] has been killed.  \e[0m"
        # 关闭程序        
        local pids=`/sbin/pidof "${t_prog_name}"`
        local pids_len=${#pids[n]}
        if [ ${pids_len} -gt 0 ];then
            # kill prog
            for pid in ${pids};
            do
                local exe=$(ls -l /proc/${pid}/exe | awk -F"exe -> " '{print $2}')
                if [ "${exe}" == "${t_prog_path}" ]; then
                    echo -e "\e[31;1m [ ${exe} ] has been killed, It's pid is: ${pid}.  \e[0m"
                    kill -9 ${pid}
                fi
            done
        else
            # kill *.sh
            ps -ef | grep -v grep | grep -v daemon | grep ${t_prog_path} | awk '{print $2}' |  xargs kill -9 >/dev/null 2>&1
            echo -e "\e[31;1mthe process [ ${t_prog_path} ] has been killed.  \e[0m"
        fi
    else
        echo -e "\e[31;1mthe process [ ${t_prog_path} ] is not running. \e[0m"
    fi
}

function show_all_matched_pid() {
# 显示所有匹配到的进程.
    local t_prog_path=$1
    local t_prog_name=$(basename ${t_prog_path})
    # daemon
    local d_num=`ps -ef | grep -v grep | grep daemon | grep "${t_prog_path}" | wc -l`
    if [ "${d_num}" -lt "1" ]; then
        echo -e "\e[31;1mthe daemon for command [ ${t_prog_path} ] is not running. \e[0m"
    else
        echo -e "\e[32;1m$(ps -ef | grep -v grep | grep daemon | grep "${t_prog_path}") \e[0m"
    fi
    # program
    local num=`ps -ef | grep -v grep | grep -v daemon | grep "${t_prog_path}" | wc -l`
    if [ "${num}" -lt "1" ]; then
        echo -e "\e[31;1mthe process [ ${t_prog_path} ] is not running. \e[0m"
    else
        local pids=$(/sbin/pidof ${t_prog_name})
        local pids_len=${#pids[n]}
        # echo ${pids_len}
        if [ ${pids_len} -gt 0 ];then
            #program
            for pid in ${pids};
            do
                local exe=$(ls -l /proc/${pid}/exe | awk -F"exe -> " '{print $2}')
                if [ "${exe}" == "${t_prog_path}" ]; then
                    echo -e "\e[32;1m$(ls -l /proc/${pid}/exe)\e[0m    [CurrentDir]"
                else
                    echo -e "\e[32;1m$(ls -l /proc/${pid}/exe) \e[0m"
                fi
            done
        else
            # shell
            ps -ef | grep -v grep | grep -v daemon | grep "${t_prog_path}"
        fi
    fi
}

parameter=$1
PARAMS=("start" "stop" "status" "show" "restart")
#for PARAM in ${PARAMS[*]};do echo $PARAM; done
USAGE="Usage: ./run.sh <${PARAMS[*]}>"
[ -z "${parameter}" ] && echo -e "\e[31;1m parameter needed, ${USAGE} \e[0m"  && exit 1
while :
do
    for PARAM in ${PARAMS[@]};do
        if [ "${parameter}" == ${PARAM} ];then
            break 2     #跳出2层循环
        fi
    done
    echo -e "\e[31;1m invalid parameter name [ $parameter ], ${USAGE} \e[0m"
    exit 1
done


DEAMON_SCRIPT="${work_dir}/daemon.sh"
INTERVAL="10s"
DAEMON_OUTPUT="${work_dir}/output$(date '+%Y%m%d').log"
COMMAND="${prog_path}"

if [ "${parameter}" == "stop" ];then
    echo -e "$(date '+%Y%m%d-%H:%M:%S') the process: [ ${COMMAND} ] manual stop" >> "${DAEMON_OUTPUT}"
    stop_daemon ${COMMAND}
fi

if [ "${parameter}" == "start" ];then
    echo -e "$(date '+%Y%m%d-%H:%M:%S') the process: [ ${COMMAND} ] manual start" >> "${DAEMON_OUTPUT}"
    start_daemon ${DEAMON_SCRIPT} ${COMMAND} ${INTERVAL} ${DAEMON_OUTPUT}
fi

if [ "${parameter}" == "restart" ];then
    echo -e "$(date '+%Y%m%d-%H:%M:%S') the process: [ ${COMMAND} ] manual restart" >> "${DAEMON_OUTPUT}"
    stop_daemon ${COMMAND}
    start_daemon ${DEAMON_SCRIPT} ${COMMAND} ${INTERVAL} ${DAEMON_OUTPUT}
fi

if [ "${parameter}" == "status" ] || [ "${parameter}" == "show" ];then
    # ps -ef | grep -v grep | grep ${prog_name}
    show_all_matched_pid "${COMMAND}"
fi
